default_color = (58275, 0, 33968, 3500)  # roughly 50% brightness at 3500k
bedtime_color = (58275, 0, 18010, 2000)  # roughly 25% brightness at 2000k
theatre_color = (58275, 0, 1, 2000)  # minimum brightness at 2000k
